function GraphWin(action)

%	<>=---------------------------------------------------------=<>
%	<>                GUI for plotting utilities                 <>
%	<>=---------------------------------------------------------=<>
% This GUI allows to create/manipulate figure and plot various 
% representation of any variable of the workspace 
% (time, FFT's angle and modulus (dB), spectrogram.
% A editable text box interprets command, useful for quick creation
% of subplot (or if you can't find your Matlab window)
% 
% The upper popup menus allow to choose a figure and its subplots
% to draw in.
% Buttons performs usual tasks : 
% Toggle GRID on/off, HOLD on/off, CLF clear figure, CLA clear axis
% POS tiles all the opened graphs in the screen
% ORIG untiles them all.
%
% The rest should be self explainatory or clear (I hope so)
% French documentation above is more documented
% 
%	<>=---------------------------------------------------------=<>
%	<>              Interface graphique pour Matlab              <>
%	<>=---------------------------------------------------------=<>
%
% Les 2 premiers PopUp permettent de choisir une figure
% et un de ses subplots. Le subplot devients alors actif
% pour les autres boutons :
%
%	- GRID : Ajoute/supprime la grille dans le subplot	
%	- HOLD : Passe de HOLD ON  OFF
%	- CLF  : Clear figure
%	- CLA  : N'efface que la courbe relative au subplot actif
%
%	- POS : Positionne toute les figures dans l'cran afin 
%		 qu'elles soient visibles
%	- ORIG : Repositionne toutes les figures  la place attibue par Matlab
%		  l'ouverture d'une figure (utile pour l'exportation de figures
%		 vers d'autres formats comme repre de taille pour les lgendes et titres)
%
%	- CLOZ FIG : Ferme la fentre slectionne
%	- NEW FIG  : Ouvre une nouvelle fentre.
%	
%	! Rq ! : Les boutons POS, ORIG, CLOZ FIG et NEW FIG, mettent aussi 
%		  jour les variables de l'espace de travail dans 
%
%	'type command' : Zone ditable o l'on peut taper une commande
%			 (plot, subplot, ...)
%
%    CADRE pour dessiner une variable contenant
%	2 menus popup :
%	   1.Menu POPUP pour choisir si l'on veut voir : 
%		- la variable
%		- sa partie relle
%		- sa partie imaginaire
%		- son module
%		- son module en dB
%		- le module de sa FFT en dB
%		- la phase de sa fft (droule au mieux)
%		- le spectrogramme de la variable (la taille des fentres
%		  est fixe  1/32 de la longueur du signal, pourquoi pas ?).
%		
%	   2.Menu POPUP contenant les variables prsentent dans l'espace de travail
%		- Seules les variables de type 'DOUBLE' sont prsentes, 
%		  avec leurs tailles.
%		- Le choix d'une variable la trace dans la figure en cours.
%		
%		N.B. : 2 variables traces successivement avec un hold on
%			seront de couleurs diffrentes
%
%	CLOSE : ferme l'assistant graphique.
%	INFO  : ouvre une fentre d'aide
%
% Pb rescencs :1. Les lgendes sont caches par la commande grid
%			-> fix le 23-08-2000.
%		2. Les subplots ne sont pas toujours  jour
%			-> vrifier  chaque opration que le subplot actif
%			   est celui que vous voulez.
%		3. Ne pas ouvrir 2 fois de suite le programme ;)
%	
%

persistent FigHndl figNumber AxeHndl editHndl listHndl varstylHndl ...
	   fenetres subplots PosPlot WinWd Screen  color VarName frmBtm;
 

if nargin<1,
    action='initialize';
end

colorstr = {'b','r','k','g','c','y','m'};

 %====================================
 % Information for all buttons - frame
 
	labelColor=[0.9 0.9 0.9];
   top=1;
   left=0.05;
   BtnWid=0.9;
   BtnHt=0.08;
  
   frmBorder=0.02;
   frmWid = BtnWid+2*frmBorder;
   frmHt = 2*BtnHt+2*frmBorder+(.1-BtnHt);



 if strcmp(action,'initialize'),
    
     % Initialisation de la fentre
    set(0,'Units','normalized')
    Screen = get(0,'ScreenSize');
    
    % Corrections pour ne pas que la fentre sorte de l'cran
    % faite manuellement (_o_) 
    xcor = 0.0039;
    ycor = -0.0234;
    WinHt = Screen(4)/3;
    WinWd = Screen(3)/10;
    hspace = Screen(3)/100;
    figNumber=figure( ...
	'Name','G@G', ...
	'NumberTitle','off', ...
        'Units','normalized', ...
	'Backingstore','off', ...
        'Position',[0+xcor  Screen(4)*2/3+ycor WinWd WinHt ], ...
        'Visible','on', ...
        'MenuBar','none');
  
	%===================================
	% The Current Figure switch Pop-Menu
	BtnNb = 1;
	btnPos=[ left top-BtnNb/10 BtnWid BtnHt];
   
   	callbackStr='GraphWin(''sousplot'');';
	
	fenetres = get(0,'children');
	fenetres(find(fenetres == figNumber)) = [];
	fenetres = sort(fenetres);
	labelStr= '';
 	for k = 1:length(fenetres)
		labelStr=[ labelStr '|Fig ' num2str(fenetres(k)) ];
	end
	labelStr = labelStr(2:end);

	figure(figNumber)
	FigHndl=uicontrol( ...
	'Style','popupmenu', ...
	'Units','normalized', ...
	'BackgroundColor',labelColor,...
	'Position',btnPos, ...
	'String',labelStr, ...
	'Interruptible','on', ...
	'Callback',callbackStr);
   
     
   %====================================
   % The GRID & HOLD button
	BtnNb = 3;
	btnPos=[ left top-BtnNb/10 BtnWid/2-0.005 BtnHt];
	    
   	labelStr='Grid';
	callbackStr='GraphWin(''togglegrid'');';
	
	GridHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos, ...
        'Enable','on', ...
        'String',labelStr, ...
	'Callback',callbackStr);

    %====================================
    % The hold button
 
	btnPos=[ left+BtnWid/2+0.005 top-BtnNb/10 BtnWid/2-0.005 BtnHt];

	labelStr='Hold';
	callbackStr='GraphWin(''holdfig'');';
	clfHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);




    %====================================
    % The CLF  & CLA buttons
        
    BtnNb = BtnNb + 1;
	btnPos=[ left top-BtnNb/10 BtnWid/2-0.005 BtnHt];

	labelStr='CLF';
	callbackStr='GraphWin(''clearfig'');';
	clfHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);


      %====================================
      % The CLA button
      
    	btnPos=[ left+BtnWid/2+0.005 top-BtnNb/10 BtnWid/2-0.005 BtnHt];

	labelStr='CLA';
	callbackStr='GraphWin(''clearaxes'');';
	claHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);

    %====================================
    % The POS and POSORIG buttons
    
    BtnNb = BtnNb + 1;
	 btnPos=[ left top-BtnNb/10 BtnWid/2-0.005 BtnHt];

    labelStr='Pos';
	 callbackStr='GraphWin(''posfig'')';
    
    posHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);
     
       %====================================
       % The reposition as original button
    
	 btnPos=[ left+BtnWid/2+0.005 top-BtnNb/10 BtnWid/2-0.005 BtnHt];

     labelStr='Orig';
	 callbackStr='GraphWin(''posorig'')';
    
     posHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);
     
    %====================================
    % The CLOSE & NEW Figure buttons
       
    BtnNb = BtnNb + 1;
	 btnPos=[ left top-BtnNb/10 BtnWid/2-0.005 BtnHt];
   
   labelStr='Cloz Fig';
   
	callbackStr='GraphWin(''closefig'')';
	closeHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);       

       %====================================
       % The NEW Figure button
	
       btnPos=[ left+BtnWid/2+0.005 top-BtnNb/10 BtnWid/2-0.005 BtnHt];

       labelStr='New Fig';
	 callbackStr='GraphWin(''newfig'')';
    
       posHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);
     
    %====================================
    % The EDIT AREA (<=> PROMPT)
       
	BtnNb = BtnNb + 1;
     	btnPos=[ left top-BtnNb/10 BtnWid BtnHt];
       labelStr='type command';
	callbackStr='GraphWin(''commandstr'')';
	editHndl=uicontrol( ...
        'Style','edit', ...
        'Units','normalized', ...
         'BackgroundColor',[1 1 1],...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);       
 
    %====================================
    % The CONSOLE frame
    % Use the frame's userdata to hold the tones matrix

    BtnNb = BtnNb + 1;

    frmBtm = top-(BtnNb+1)/10-frmBorder;
    frmPos=[left-frmBorder frmBtm frmWid frmHt ];
    
    frmHndl=uicontrol( ...
        'Style','frame', ...
        'Units','normalized', ...
        'Position',frmPos, ...
        'BackgroundColor',[0.75 0.750 0.750]);

      
       %====================
       % FRAME TITLE
	labelStr='Style & var to plot';
	btnPos=[ left frmBtm+2*frmHt/3 BtnWid frmHt/3.7];
    
     uicontrol( ...
        'Style','text', ...
        'Units','normalized', ...
        'Position',btnPos, ...
        'HorizontalAlignment','center', ...
        'BackgroundColor',[0.75 0.750 0.750],...
        'String',labelStr);
   
      
      
       %================================================
       % POPUP VARstyle (real, imag, module, module DB, fftdb)
	
	btnPos=[ left frmBtm+frmBorder+frmHt/3 BtnWid frmHt/3.7];
	
   	callbackStr='GraphWin(''varstyle'');';
	
	labelStr = 'Variable|Real|Imag|Module|Phase|Module_db|Abs fft dB|Phi fft|Spectrogram';

	figure(figNumber)
	
	varstylHndl=uicontrol( ...
	'Style','popupmenu', ...
	'Units','normalized', ...
	'BackgroundColor',labelColor,...
	'Position',btnPos, ...
	'String',labelStr, ...
	'Interruptible','on', ...
	'Callback',callbackStr);
   

    %====================================
    % The LIST OF VARIABLES to plot button
       
    BtnNb = BtnNb + 1;
	btnPos=[ left frmBtm+frmBorder BtnWid frmHt/3.7];
	
	color = 1;	% pointeur sur les couleurs
	
	variables = evalin('base','whos');
	labelStr='';   
	for k = 1:length(variables)
		sizevar = variables(k).size;
		sizstr = '';
		for l = 1:length(sizevar)
			sizstr = [ sizstr num2str(sizevar(l)) 'x' ];
		end
		sizstr = sizstr(1:end-1);
		
		if strcmp(variables(k).class,'double') & max(sizevar)>1
			labelStr=[ labelStr '|' variables(k).name ' : ' sizstr ];
		end
	end
	labelStr = labelStr(2:end);


	callbackStr='GraphWin(''listvar'')';
	listHndl=uicontrol( ...
        'Style','popup', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);       
 
     
     
    %====================================
    % The CLOSE & HELP buttons

    BtnNb = BtnNb + 1;
	btnPos=[ left top-BtnNb/10 BtnWid/2-0.005 BtnHt-0.01];
   
       labelStr='Close';
		callbackStr='close(gcf)';
	closeHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);       
 

       %====================================
       % The INFO button

	btnPos=[ left+BtnWid/2+0.005 top-BtnNb/10 BtnWid/2-0.005 BtnHt-0.01];
   
	labelStr='Help';
		callbackStr='helpwin(''GraphWin'')';
	infoHndl=uicontrol( ...
        'Style','push', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);       
 
 
 	btnPos=[ left+BtnWid/2+0.005 top-BtnNb/10 BtnWid/2-0.005 BtnHt];

     

   GraphWin('actualize');

% End of Initialization
     
%=================================================================
% 			ACTIONS
%=================================================================

%=======================
% Mise  jour des pop-up
elseif strcmp(action,'actualize'),
     
	%===================================
	% The Current Figure switch Pop-Menu
	BtnNb = 1;
	btnPos=[ left top-BtnNb/10 BtnWid BtnHt];
   
   	if ~isempty(FigHndl)
   		CurrentFigHndl = get(FigHndl,'value');
   	else
   	   CurrentFigHndl = 1;
   	end
   
   callbackStr='GraphWin(''sousplot'');';
	
	fenetres = get(0,'children');
	fenetres(find(fenetres == figNumber)) = [];
	fenetres = sort(fenetres);
	labelStr= '';
 	for k = 1:length(fenetres)
		labelStr=[ labelStr '|Fig ' num2str(fenetres(k)) ];
	end
	labelStr = labelStr(2:end);

	figure(figNumber)
	FigHndl=uicontrol( ...
	'Style','popupmenu', ...
	'Units','normalized', ...
	'BackgroundColor',labelColor,...
	'Position',btnPos, ...
	'String',labelStr, ...
	'Interruptible','on', ...
	'Callback',callbackStr);


	%====================================
	% on lance la recherche des subplots
	
	% On se remet dans la figure de dessin pour continuer
	set(FigHndl,'value',CurrentFigHndl);
	 
	GraphWin('sousplot');



%===============================
% Recherche des subplots

elseif strcmp(action,'sousplot'),
	BtnNb = 2;
	btnPos=[ left top-BtnNb/10 BtnWid BtnHt];
	
	if ~isempty(fenetres)
	 	figure(fenetres(get(FigHndl,'value')))
	   
		childfig = sort(get(gcf,'children'));
	   
		subplots = [];
		PosPlot = [];
		l = 0;
	    
		for k = 1:length(childfig)
			axes(childfig(k))
			if length(get(gca,'DeleteFcn')) < 2 	% dessin
				l = l+1;
	        		subplots = [subplots childfig(k) ];
	        		PosPlot(l,1:4) = get(gca,'Position');
			end
		end
		labelStr= '';
   		for k = 1:l
   			labelStr=[ labelStr '|subplot ' num2str(k) ];
		end
		labelStr = labelStr(2:end);

	else

		l=0;

		labelStr= ' ';

		disp('Pas de figures ouvertes')

	end

	%==================
   	% Bouton
   	figure(figNumber)

	
	AxeHndl=uicontrol( ...
	'Style','popupmenu', ...
	'Units','normalized', ...
	'BackgroundColor',labelColor,...
	'Position',btnPos, ...
	'String',labelStr, ...
	'Interruptible','on');

	if ~isempty(fenetres)
		figure(fenetres(get(FigHndl,'value')))
	   	if ~isempty(get(gcf,'children'))
			axes(subplots(get(AxeHndl,'value')));
			legend
		end		
	end
   
%=============================
% 	GRID ON / OFF
elseif strcmp(action,'togglegrid'),
	
	figure(fenetres(get(FigHndl,'value')))
	if ~isempty(get(gcf,'children'))
	   axes(subplots(get(AxeHndl,'value')))
	   grid;
	else
	   grid;
	end
	GraphWin('actualize')

%=============================
% 	HOLD ON / OFF
elseif strcmp(action,'holdfig'),
	
	figure(fenetres(get(FigHndl,'value')))
	if ~isempty(get(gcf,'children'))
		axes(subplots(get(AxeHndl,'value')))
   	hold;
   else
      hold
   end
   GraphWin('actualize')
   
   
%=============================
% 	CLEAR FIGURE
elseif strcmp(action,'clearfig'),
	
	figure(fenetres(get(FigHndl,'value')))
	clf;
   GraphWin('actualize')

%=============================
% 	CLEAR AXES
elseif strcmp(action,'clearaxes'),
	
	figure(fenetres(get(FigHndl,'value')))
	if ~isempty(get(gcf,'children'))
		axes(subplots(get(AxeHndl,'value')))
   	cla;
   else
      clf;
   end
%    GraphWin('actualize')





%================================
% 	POSITONNEMENT DES FIGURES
elseif strcmp(action,'posfig'),
	position(WinWd+0.005,Screen(4)/40,fenetres);
	GraphWin('actualvar')

   
%=============================
% 	REPOSITIONNMENT ORIGNAL
elseif strcmp(action,'posorig'),
   h = figure;
	set(h,'Units','normalized')
	pos = get(h,'Position');
	close(h)

	n_fig = length(fenetres);
   
	for k= 1 : n_fig
	   set(fenetres(k),'Units','normalized')
	   figure(fenetres(k));
		set(fenetres(k),'Position',pos);
	end
	GraphWin('actualvar')
   
   
  
 %=============================
 % 	CLOSE FIGURE
 elseif strcmp(action,'closefig'),
   if ~isempty(fenetres)
	    numfig =  get(FigHndl,'value');
	    figure(fenetres(numfig)),
	    close
	   if numfig~=1
		figure(fenetres(numfig-1)),       
	    	set(FigHndl,'value',numfig-1)
	   end
    	   GraphWin('actualvar')	   
	   GraphWin('actualize')
   else
   	disp('No more Figures to close !')
   end
   
 %=============================
 % NEW FIGURE
 elseif strcmp(action,'newfig'),   
    figure
    grid
    GraphWin('actualvar')

   GraphWin('actualize')




    
 %=============================
 % Evaluate EDITABLE TEXT
 elseif strcmp(action,'commandstr'),   
   
    	if ~isempty(fenetres)
		figure(fenetres(get(FigHndl,'value')))
		if ~isempty(get(gcf,'children'))
			axes(subplots(get(AxeHndl,'value')))
		end
	else
		figure
	end
  cmd = get(editHndl,'String');
   
   evalin('base',cmd)
   GraphWin('actualize')   



 %=============================
 % VARIABLE STYLE CHOICE.
 elseif strcmp(action,'varstyle'),
 	VarName = get(listHndl,'string');
	VarName = VarName(get(listHndl,'value'),:);
  	VarName = VarName(1:find(VarName==':')-1);
  	
  	nb = get(varstylHndl,'value');
  	if nb == 2
  		VarName = ['real(' VarName ')' ];
  	elseif nb == 3
  		VarName = ['imag(' VarName ')' ];
  	elseif nb == 4
  		VarName = ['abs(' VarName ')' ];
  	elseif nb == 5
  		VarName = ['angle(' VarName ')' ];
  	elseif nb == 6
  		VarName = ['20*log10(abs(' VarName '))' ];
  	elseif nb == 7
  		VarName = ['20*log10(abs(fft(' VarName ')))' ];
   	elseif nb == 8
  		VarName = ['unwrap(angle(fft(' VarName ')))' ];
   	elseif nb == 9
   		nfft = evalin('base',[ 'length(' VarName ')' ]);
   		nfft = max(16,2^nextpow2(nfft/32));
   		VarName = ['specgram(' VarName ',' num2str(nfft) ')' ];
	end
   	if ~isempty(fenetres)
		figure(fenetres(get(FigHndl,'value')))
		if ~isempty(get(gcf,'children'))
			axes(subplots(get(AxeHndl,'value')))
			if strcmp(get(gca,'NextPlot'),'add')
				color = color + 1;
			else
				color = 1;
			end
		end
	else
		figure
		color = 1;
  	end

    if get(varstylHndl,'value')~=9
	   	couleur = colorstr{rem(color,8)+(rem(color,8)==0)};	
		cmd = [ 'plot(' VarName ','''  couleur ''')'];
	else
		cmd = VarName;
	end
	
	evalin('base',cmd)  	
	  	
  	GraphWin('actualize')	

    
 %=============================
 % VARIABLE LIST CHOICE.
 elseif strcmp(action,'listvar'),
 
   	if ~isempty(fenetres)
		figure(fenetres(get(FigHndl,'value')))
		if ~isempty(get(gcf,'children'))
			axes(subplots(get(AxeHndl,'value')))
			if strcmp(get(gca,'NextPlot'),'add')
				color = color + 1;
			else
				color = 1;
			end
		end
	else
		figure
		color = 1;
  	end
  	
 	GraphWin('varstyle')
	if get(varstylHndl,'value')~=9
	   	couleur = colorstr{rem(color,8)+(rem(color,8)==0)};	
		cmd = [ 'plot(' VarName ','''  couleur ''')'];
	else
		cmd = VarName;
	end
	
	evalin('base',cmd)  	
	  	
  	GraphWin('actualize')	


%====================================================
% actualisation des variables de l'espace de travail

elseif strcmp(action,'actualvar'),
	
   	if ~isempty(FigHndl)
   		CurrentFigHndl = get(FigHndl,'value');
   	else
   	   CurrentFigHndl = 1;
   	end
   
	figure(figNumber)
	

	btnPos=[ left frmBtm+frmBorder BtnWid frmHt/3.7];


	currentvar = get(listHndl,'string');
 	varnumber = get(listHndl,'value');
	currentvar =currentvar(varnumber,:);
	
	variables = evalin('base','whos');
	labelStr='';   
	for k = 1:length(variables)
		sizevar = variables(k).size;
		sizstr = '';
		for l = 1:length(sizevar)
			sizstr = [ sizstr num2str(sizevar(l)) 'x' ];
		end
		sizstr = sizstr(1:end-1);
		
		if strcmp(variables(k).class,'double') & max(sizevar)>1
			labelStr=[ labelStr '|' variables(k).name ' : ' sizstr ];
		end
	end
	labelStr = labelStr(2:end);

	callbackStr='GraphWin(''listvar'')';
	listHndl=uicontrol( ...
        'Style','popup', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos,...
        'String',labelStr, ...
        'Callback',callbackStr);       
 
 	newvar = get(listHndl,'string');
 	
 	[nvar,lmaxvar] = size(newvar);
 	lcurvar = length(currentvar);
 	
 	isitin = [];
 	for k = 1:nvar
	 	isitin = [ isitin strncmp(newvar(k,:),currentvar,min(lcurvar,lmaxvar)) ];
 	end
 	
 	if sum(isitin)
 		valvar = find(isitin);
 		set(listHndl,'value',valvar(1));
 	end
 	
	% On se remet dans la figure de dessin pour continuer
	set(FigHndl,'value',CurrentFigHndl);





%=================

% Fin du programme

end