function visudsp(varargin)

%Function visudsp(signal1,signal2,...)
% Plots the PSD of the given input signals (vectors)
% with normalized frequency range [0,pi]
% Signals are assumed to have the same sampling frequency
% Uses different color (6) for any signal
%
% Affichage de la DSP d'un signal (priodogramme)
% dans une nouvelle figure
%	Entres:   signal1, signal2,... : signaux temporels
%
%SEE ALSO: visudspind,visusigspectro,visuspectro

% Cellule contenant les couleurs
% Pour avoir les courbes en noir et blanc, 
% 6 types de traits diffrents
% intervertir les deux lignes ci-dessous.
%couleur= {'k','k-.','k--','ko','k+-','k:'};
couleur= {'b','r','g','k','y','m'};


figure
hold on;
title('DSP en dB entre 0 et pi (fe/2)')

set(gca,'XTick',0:pi/4:2*pi)
set(gca,'XTickLabel',{'0','pi/4','pi/2','3pi/4','pi',...
	'5pi/4','3pi/2','7pi/4','2*pi'})
grid on

nb_filt = length(varargin);

for k = 1:nb_filt
	N = length(varargin{k});
	% Axe des pulsations
	axf = (0:N-1)*2*pi/N;
	
	% Densit spectrale de puissance ( 1/sqrt(2)^2 = 1/N )
	period = 1/N*(abs(fft(varargin{k}))).^2;  
	
	% Visualisation
	plot(axf(1:floor(N/2)+1),10*log10(period(1:floor(N/2)+1)+1e-16),couleur{rem(k-1,7)+1})
	xlabel('\omega normalis')
	ylabel('dB')
end

position;
