function visufilt(varargin)

%Function visufilt(IR_filtre1,IR_filtre2,...)
% Plots in a new figure the modulus and angle of 
% input Impulse responses of filters (e.g. filter banks IR)
%
% Affichage du module et de la phase de filtres
% dans une nouvelle figure
%	Entres:   filtre1, filtre2,... : RI des filtres
% Remarque : il y a 7 couleurs diffrentes
%
% Example:
%   L_filt = 6;
%   IR_1 = 1/L_filt*ones(1,L_filt);
%   IR_2 = (-1).^(1:L_filt).*IR_1;
%   visufilt(IR_1,IR_2)
%
%SEE ALSO : visufiltNoWin
%

N = 0;
nb_filt = length(varargin);
for k = 1:nb_filt
    N = max(N,length(varargin{k}));
end    
N = max(N,512);

% Cellule contenant les couleurs
couleur= {'b','r','k','g','c','y','m'};

% Axe des pulsations
axf = (1:N)*2*pi/N;

figure
subplot(211)
hold on;
title('Module en dB')
set(gca,'XTick',0:pi/4:2*pi)
set(gca,'XTickLabel',{'0','pi/4','pi/2','3pi/4','pi'})
grid on

subplot(212)
hold on;
title('Phase droule au mieux')
set(gca,'XTick',0:pi/4:2*pi)
set(gca,'XTickLabel',{'0','pi/4','pi/2','3pi/4','pi'})
grid on

nb_filt = length(varargin);

for k = 1:nb_filt
	module = abs(fft(varargin{k},N));
	phase = unwrap(angle(fft(varargin{k},N)));
	subplot(211)
	plot(axf(1:floor(N/2)+1),20*log10(module(1:floor(N/2)+1)+1e-16),couleur{rem(k-1,7)+1})
	subplot(212)
	plot(axf(1:floor(N/2)+1),phase(1:floor(N/2)+1),couleur{rem(k-1,7)+1})
end

