function visusignal(varargin)

%Function visusignal(signal_1,signal_2,...,'nb')
%
% Plots the time waveforms of input signals.
% An optional string argument 'nb' can be put at the end
% to have black and white plot
% Allows up to 6 colors or draw types.
%
% Affichage de la forme d'onde et du module des filtres
% dans une nouvelle figure
%	Entres:   signal1, signal2,... : RI des filtres
%		'nb' : variable optionnelle pour avoir les courbes
%			en noir et blac
% Remarque : 6 types de traits diffrents
%
%SEE ALSO: visudsp, visudspind, visusigspectro, visuspectro
%
N = [];

for k = 1 : nargin
	N = [ N length(varargin{k}) ];	
end
N = max(N)*2;

nb_filt = length(varargin);

% Cellule contenant les couleurs
couleur= {'b','r','g','k','y','m','c'};

if strcmp(varargin{end},'nb')
	couleur = {'k','k-.','k--','ko','k+-','k:'};
	nb_filt = length(varargin)-1;
end

% Axe des pulsations
axf = (0:N-1)/N;

figure
clf
subplot(2,1,1)
hold on;
title('Allure temporelle (en chantilllons)')
subplot(2,1,2)
hold on;
title('Module en dB')
set(gca,'XTick',0:0.1:1)
set(gca,'XTickLabel',{'0','.1','.2','.3','.4',...
	'Fe/2','.6','.7','.8','.9','1=Fe'})

for k = 1:nb_filt
	modul = abs(fft(varargin{k},N));

	subplot(2,1,1)
	if ~isreal(varargin{k})
		plot(real(varargin{k}),couleur{rem(k-1,7)+1})
		plot(imag(varargin{k}),[couleur{rem(k-1,7)+1} '.-'])
	else
		plot(varargin{k},couleur{rem(k-1,7)+1})
	end
	subplot(2,1,2)
	plot(axf,20*log10(modul),couleur{rem(k-1,7)+1})
end

